<?php

namespace Drupal\Tests\url_friendly_options\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Test the overrides on the Selection List field configuration form.
 *
 * @group url_friendly_options
 */
class SelectFieldConfigFormTest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'field_ui',
    'user',
    'url_friendly_options',
  ];

  /**
   * The user to use during testing.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->createContentType([
      'type' => 'article',
      'name' => 'Article',
    ]);

    $this->adminUser = $this->drupalCreateUser([
      'administer content types',
      'administer node fields',
      'create article content',
    ]);

  }

  /**
   * Test the auto-generated machine names.
   */
  public function testSelectFieldConfigForm() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('admin/structure/types/manage/article/fields/add-field');

    $page = $this->getSession()->getPage();
    $page->selectFieldOption('new_storage_type', 'selection_list');
    $page->pressButton('Continue');
    $this->assertStringContainsString('Add field', $page->getText());
    $page->fillField('label', 'List');
    $page->selectFieldOption('group_field_options_wrapper', 'list_string');
    $page->pressButton('Continue');
    $this->assertStringContainsString('List settings for Article', $page->getText());

    // Test for string with spaces.
    $page->fillField('field_storage[subform][settings][allowed_values][table][0][item][label]', ' string with spaces ');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $this->assertStringContainsString('string-with-spaces', $page->findAll('css', '.machine-name-value')[0]->getText());

    // Test for string with underscores.
    $page->pressButton('Add another item');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $page->fillField('field_storage[subform][settings][allowed_values][table][1][item][label]', '_string_with_underscores_');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $this->assertStringContainsString('string-with-underscores', $page->findAll('css', '.machine-name-value')[1]->getText());

    // Test for string with special characters.
    $page->pressButton('Add another item');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $page->fillField('field_storage[subform][settings][allowed_values][table][2][item][label]', '<?php echo "Hello world"; ?>');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $this->assertStringContainsString('php-echo-hello-world', $page->findAll('css', '.machine-name-value')[2]->getText());

    // Test for string with numbers.
    $page->pressButton('Add another item');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $page->fillField('field_storage[subform][settings][allowed_values][table][3][item][label]', 'string with numbers 1234567890');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $this->assertStringContainsString('string-with-numbers-1234567890', $page->findAll('css', '.machine-name-value')[3]->getText());

    // Save list options.
    $page->pressButton('Save settings');
    $this->assertStringContainsString('Saved List configuration', $page->getText());

    // Verify article content type has new field and options created above.
    $this->drupalGet('node/add/article');
    $page->selectFieldOption('field_list', 'string-with-spaces');
    $page->selectFieldOption('field_list', 'string-with-underscores');
    $page->selectFieldOption('field_list', 'php-echo-hello-world');
    $page->selectFieldOption('field_list', 'string-with-numbers-1234567890');
  }

  /**
   * Test the form validation.
   */
  public function testSelectFieldConfigFormValidation() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('admin/structure/types/manage/article/fields/add-field');

    $page = $this->getSession()->getPage();
    $page->selectFieldOption('new_storage_type', 'selection_list');
    $page->pressButton('Continue');
    $this->assertStringContainsString('Add field', $page->getText());
    $page->fillField('label', 'Options');
    $page->selectFieldOption('group_field_options_wrapper', 'list_string');
    $page->pressButton('Continue');
    $this->assertStringContainsString('Options settings for Article', $page->getText());

    // Edit the auto-generated machine-name by replacing the hyphen
    // with an underscore, which is not a character allowed.
    $page->fillField('Name', 'Option 1');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $page->pressButton('Edit');
    $page->fillField('Value', 'option_1');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $page->pressButton('Save settings');
    $this->assertStringContainsString('The following keys are not URL-friendly: option_1', $page->getText());
  }

  /**
   * Make sure Boolean fields are not affected.
   *
   * Prevents regression of #3527940.
   */
  public function testBooleanField() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('admin/structure/types/manage/article/fields/add-field');

    $page = $this->getSession()->getPage();
    $page->selectFieldOption('new_storage_type', 'boolean');
    $page->pressButton('Continue');
    $this->assertStringContainsString('Add field', $page->getText());
    $page->fillField('label', 'Boolean');
    $page->pressButton('Continue');
    $this->assertStringContainsString('Boolean settings for Article', $page->getText());
  }

}
